#!/usr/bin/python
# -*- coding: utf-8 -*-

import os

root_path = os.path.abspath(os.path.dirname(__file__))
data_path = os.path.join(root_path, 'data')
homophone_path = os.path.join(data_path, '同音意字.txt')
Equivalent_char_path = os.path.join(data_path, '等价字.txt')
similarword_path = os.path.join(data_path, '同义词.txt')
company_path = os.path.join(data_path, 'company.txt')
random_path = os.path.join(data_path, 'company.txt')

